@@ -241,7 +241,7 @@ void mir_output_set_pixel_format(MirOutput* output, MirPixelFormat format);
  * Get the ID of an output
  *
  * This can be used to refer to the output in other parts of the API, such as
- * mir_surface_spec_set_fullscreen_on_output().
+ * mir_window_spec_set_fullscreen_on_output().
  *
  * \param [in]  output  The MirOutput to query.
  * \returns     The ID of output, which may be used to refer to it in other
@@ -266,7 +266,8 @@ MirOutputType mir_output_get_type(MirOutput const* output);
  * \param [in]  type  The MirDisplayOutputType to describe.
  * \returns           The name of the output type.
  */
-char const* mir_display_output_type_name(MirDisplayOutputType type);
+char const* mir_display_output_type_name(MirDisplayOutputType type)
+__attribute__((deprecated("use mir_output_type_name instead")));
 
 /**
  * Get the textual name of an output type.
@@ -372,6 +373,18 @@ void mir_output_enable(MirOutput* output);
  */
 void mir_output_disable(MirOutput* output);
 
+/**
+ * Get a descriptive manufacturer/model string for the connected display.
+ * The format of this string is arbitrary and driver-specific but should be
+ * human-readable and helpful for someone to identify which physical display
+ * this is. Note this function is not called get_name because that would imply
+ * the returned value is different for each output, whereas it may not be.
+ *
+ * \returns  A nul-terminated string or NULL if none available. This string
+ *           remains valid for the lifetime of the MirOutput object.
+ */
+char const* mir_output_get_model(MirOutput const* output);
+
 /**
  * Get the physical width of the connected display, in millimetres.
  *
@@ -519,6 +532,45 @@ void mir_output_set_gamma(MirOutput* client_output,
                           uint16_t const* blue,
                           uint32_t  size);
 
+/**
+ * Set the scale-factor of a display
+ *
+ * The scale-factor specifies the conversion between logical pixels and physical
+ * pixels on this output. See mir_output_get_scale_factor for further details.
+ *
+ * \param [in]  output  The MirOutput to modify
+ * \param [in]  scale   The scale factor
+ */
+void mir_output_set_scale_factor(MirOutput* output, float scale);
+
+/**
+ * Get the raw EDID data of a display.
+ *
+ * This returns a pointer to the start of the raw, unparsed EDID data.
+ * Some displays or connection types may not provide EDID data. In that case,
+ * this returns NULL.
+ *
+ * An EDID is always at least 128 bytes, but may be longer in the presence of extensions.
+ *
+ * \param [in]  output  The MirOutput to query
+ * \returns     A pointer to the start of the raw EDID data.
+ *              This pointer remains valid as long as the MirOutput remains valid.
+ *              Returns NULL if there is no EDID available.
+ */
+uint8_t const* mir_output_get_edid(MirOutput const* output);
+
+/**
+ * Get the size of the EDID of this display.
+ *
+ * If the EDID is unavailable for some reason this returns 0.
+ *
+ * An EDID is always at least 128 bytes, but may be longer in the presence of extensions.
+ *
+ * \param [in]  output  The MirOutput to query
+ * \returns     The size of the data pointed to by mir_output_get_edid(output).
+ */
+size_t mir_output_get_edid_size(MirOutput const* output);
+
 /**
  * Get the width, in pixels, of a MirOutputMode
  *
