@@ -20,7 +20,7 @@
 #include "reordering_message_sender.h"
 #include "event_sink_factory.h"
 
-#include "mir/frontend/session_mediator_report.h"
+#include "mir/frontend/session_mediator_observer.h"
 #include "mir/frontend/shell.h"
 #include "mir/frontend/session.h"
 #include "mir/frontend/surface.h"
@@ -48,6 +48,11 @@
 #include "mir/frontend/prompt_session.h"
 #include "mir/frontend/buffer_stream.h"
 #include "mir/input/input_device_hub.h"
+#include "mir/input/mir_input_config.h"
+#include "mir/input/mir_input_config_serialization.h"
+#include "mir/input/mir_touchpad_config.h"
+#include "mir/input/mir_pointer_config.h"
+#include "mir/input/mir_keyboard_config.h"
 #include "mir/input/device.h"
 #include "mir/scene/prompt_session_creation_parameters.h"
 #include "mir/fd.h"
@@ -56,6 +61,7 @@
 
 #include "mir/geometry/rectangles.h"
 #include "protobuf_buffer_packer.h"
+#include "protobuf_input_converter.h"
 
 #include "mir_toolkit/client_types.h"
 
@@ -78,6 +84,13 @@ namespace geom = mir::geometry;
 
 namespace
 {
+//TODO: accept other pixel format types
+void throw_if_unsuitable_for_cursor(mf::BufferStream& stream)
+{
+    if (stream.pixel_format() != mir_pixel_format_argb_8888)
+        BOOST_THROW_EXCEPTION(std::logic_error("Only argb8888 buffer streams may currently be attached to the cursor"));
+}
+
 mg::GammaCurve convert_string_to_gamma_curve(std::string const& str_bytes)
 {
     mg::GammaCurve out(str_bytes.size() / (sizeof(mg::GammaCurve::value_type) / sizeof(char)));
@@ -91,7 +104,7 @@ mf::SessionMediator::SessionMediator(
     std::shared_ptr<mg::PlatformIpcOperations> const& ipc_operations,
     std::shared_ptr<mf::DisplayChanger> const& display_changer,
     std::vector<MirPixelFormat> const& surface_pixel_formats,
-    std::shared_ptr<SessionMediatorReport> const& report,
+    std::shared_ptr<SessionMediatorObserver> const& observer,
     std::shared_ptr<mf::EventSinkFactory> const& sink_factory,
     std::shared_ptr<mf::MessageSender> const& message_sender,
     std::shared_ptr<MessageResourceCache> const& resource_cache,
@@ -107,7 +120,7 @@ mf::SessionMediator::SessionMediator(
     ipc_operations(ipc_operations),
     surface_pixel_formats(surface_pixel_formats),
     display_changer(display_changer),
-    report(report),
+    observer(observer),
     sink_factory{sink_factory},
     event_sink{sink_factory->create_sink(message_sender)},
     message_sender{message_sender},
@@ -126,7 +139,7 @@ mf::SessionMediator::~SessionMediator() noexcept
 {
     if (auto session = weak_session.lock())
     {
-        report->session_error(session->name(), __PRETTY_FUNCTION__, "connection dropped without disconnect");
+        observer->session_error(session->name(), __PRETTY_FUNCTION__, "connection dropped without disconnect");
         shell->close_session(session);
     }
     destroy_screencast_sessions();
@@ -142,7 +155,7 @@ void mf::SessionMediator::connect(
     ::mir::protobuf::Connection* response,
     ::google::protobuf::Closure* done)
 {
-    report->session_connect_called(request->application_name());
+    observer->session_connect_called(request->application_name());
 
     auto const session = shell->open_session(client_pid_, request->application_name(), event_sink);
     weak_session = session;
@@ -172,21 +185,34 @@ void mf::SessionMediator::connect(
     auto protobuf_config = response->mutable_display_configuration();
     mfd::pack_protobuf_display_configuration(*protobuf_config, *display_config);
 
+    MirInputConfig temp;
     hub->for_each_input_device(
-        [response](auto const& dev)
+        [&temp](auto const& dev)
         {
-            auto dev_info = response->add_input_devices();
-            dev_info->set_name(dev.name());
-            dev_info->set_id(dev.id());
-            dev_info->set_unique_id(dev.unique_id());
-            dev_info->set_capabilities(dev.capabilities().value());
+            MirInputDevice conf(dev.id(), dev.capabilities(), dev.name(), dev.unique_id());
+            auto ptr_conf = dev.pointer_configuration();
+            auto tpd_conf = dev.touchpad_configuration();
+            auto kbd_conf = dev.keyboard_configuration();
+
+            if (ptr_conf.is_set())
+                conf.set_pointer_config(ptr_conf.value());
+            if (tpd_conf.is_set())
+                conf.set_touchpad_config(tpd_conf.value());
+            if (kbd_conf.is_set())
+                conf.set_keyboard_config(kbd_conf.value());
+
+            temp.add_device_config(conf);
         });
 
+    response->set_input_configuration(mi::serialize_input_config(temp));
+
     for (auto pf : surface_pixel_formats)
         response->add_surface_pixel_format(static_cast<::google::protobuf::uint32>(pf));
 
     resource_cache->save_resource(response, ipc_package);
 
+    response->set_coordinate_translation_present(translator->translation_supported());
+
     done->Run();
 }
 
@@ -221,7 +247,7 @@ void mf::SessionMediator::create_surface(
     if (session.get() == nullptr)
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
 
-    report->session_create_surface_called(session->name());
+    observer->session_create_surface_called(session->name());
 
     auto params = ms::SurfaceCreationParameters()
         .of_size(request->width(), request->height())
@@ -235,10 +261,10 @@ void mf::SessionMediator::create_surface(
         params.with_output_id(graphics::DisplayConfigurationOutputId(request->output_id()));
 
     if (request->has_type())
-        params.of_type(static_cast<MirSurfaceType>(request->type()));
+        params.of_type(static_cast<MirWindowType>(request->type()));
 
     if (request->has_state())
-        params.with_state(static_cast<MirSurfaceState>(request->state()));
+        params.with_state(static_cast<MirWindowState>(request->state()));
 
     if (request->has_pref_orientation())
         params.with_preferred_orientation(static_cast<MirOrientationMode>(request->pref_orientation()));
@@ -344,13 +370,13 @@ void mf::SessionMediator::create_surface(
     if (surface->supports_input())
         response->add_fd(surface->client_input_fd());
     
-    for (unsigned int i = 0; i < mir_surface_attribs; i++)
+    for (unsigned int i = 0; i < mir_window_attribs; i++)
     {
         auto setting = response->add_attributes();
         
         setting->mutable_surfaceid()->set_value(surf_id.as_value());
         setting->set_attrib(i);
-        setting->set_ivalue(shell->get_surface_attribute(session, surf_id, static_cast<MirSurfaceAttrib>(i)));
+        setting->set_ivalue(shell->get_surface_attribute(session, surf_id, static_cast<MirWindowAttrib>(i)));
     }
 
     if (legacy_stream)
@@ -372,7 +398,7 @@ void mf::SessionMediator::submit_buffer(
 {
     auto const session = weak_session.lock();
     if (!session) BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
-    report->session_submit_buffer_called(session->name());
+    observer->session_submit_buffer_called(session->name());
     
     mf::BufferStreamId const stream_id{request->id().value()};
     mg::BufferID const buffer_id{static_cast<uint32_t>(request->buffer().buffer_id())};
@@ -382,7 +408,7 @@ void mf::SessionMediator::submit_buffer(
     auto b = session->get_buffer(buffer_id);
     ipc_operations->unpack_buffer(request_msg, *b);
 
-    stream->swap_buffers(b.get(), [](mg::Buffer*) {});
+    stream->submit_buffer(b);
 
     done->Run();
 }
@@ -396,16 +422,38 @@ void mf::SessionMediator::allocate_buffers(
     if (!session)
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
 
-    report->session_allocate_buffers_called(session->name());
+    observer->session_allocate_buffers_called(session->name());
     for (auto i = 0; i < request->buffer_requests().size(); i++)
     {
         auto const& req = request->buffer_requests(i);
-        mg::BufferProperties properties(
-            geom::Size{req.width(), req.height()},
-            static_cast<MirPixelFormat>(req.pixel_format()),
-           static_cast<mg::BufferUsage>(req.buffer_usage()));
 
-        auto id = session->create_buffer(properties);
+        mg::BufferID id;
+        if (req.has_flags() && req.has_native_format())
+        {
+            auto session_ext = std::dynamic_pointer_cast<SessionExtensions>(session);
+            id = session_ext->create_buffer({req.width(), req.height()}, req.native_format(), req.flags());
+        }
+        else if (req.has_buffer_usage() && req.has_pixel_format())
+        {
+            auto const usage = static_cast<mg::BufferUsage>(req.buffer_usage());
+            geom::Size const size{req.width(), req.height()};
+            auto const pf = static_cast<MirPixelFormat>(req.pixel_format());
+            if (usage == mg::BufferUsage::software)
+            {
+                auto session_ext = std::dynamic_pointer_cast<SessionExtensions>(session);
+                id = session_ext->create_buffer(size, pf);
+            }
+            else
+            {
+                //legacy route, server-selected pf and usage
+                id = session->create_buffer(mg::BufferProperties{size, pf, mg::BufferUsage::hardware});
+            }
+        }
+        else
+        {
+            BOOST_THROW_EXCEPTION(std::logic_error("Invalid buffer request"));
+        }
+
         if (request->has_id())
         {
             auto stream = session->get_buffer_stream(mf::BufferStreamId(request->id().value()));
@@ -424,7 +472,7 @@ void mf::SessionMediator::release_buffers(
     if (!session)
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
 
-    report->session_release_buffers_called(session->name());
+    observer->session_release_buffers_called(session->name());
     if (request->has_id())
     {
         auto stream_id = mf::BufferStreamId{request->id().value()};
@@ -459,7 +507,7 @@ void mf::SessionMediator::release_surface(
     if (session.get() == nullptr)
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
 
-    report->session_release_surface_called(session->name());
+    observer->session_release_surface_called(session->name());
 
     auto const id = SurfaceId(request->value());
 
@@ -487,7 +535,7 @@ void mf::SessionMediator::disconnect(
         if (session.get() == nullptr)
             BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
 
-        report->session_disconnect_called(session->name());
+        observer->session_disconnect_called(session->name());
 
         shell->close_session(session);
         destroy_screencast_sessions();
@@ -502,7 +550,7 @@ void mf::SessionMediator::configure_surface(
     mir::protobuf::SurfaceSetting* response,
     google::protobuf::Closure* done)
 {
-    MirSurfaceAttrib attrib = static_cast<MirSurfaceAttrib>(request->attrib());
+    auto attrib = static_cast<MirWindowAttrib>(request->attrib());
 
     // Required response fields:
     response->mutable_surfaceid()->CopyFrom(request->surfaceid());
@@ -513,7 +561,7 @@ void mf::SessionMediator::configure_surface(
     if (session.get() == nullptr)
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
 
-    report->session_configure_surface_called(session->name());
+    observer->session_configure_surface_called(session->name());
 
     auto const id = mf::SurfaceId(request->surfaceid().value());
     int value = request->ivalue();
@@ -615,6 +663,21 @@ void mf::SessionMediator::modify_surface(
             surface_specification.max_aspect().height()
         };
 
+    if (surface_specification.has_cursor_name())
+    {
+        mods.cursor_image = cursor_images->image(surface_specification.cursor_name(), mi::default_cursor_size);
+    }
+
+    if (surface_specification.has_cursor_id() &&
+        surface_specification.has_hotspot_x() &&
+        surface_specification.has_hotspot_y())
+    {
+        mf::BufferStreamId id{surface_specification.cursor_id().value()};
+        throw_if_unsuitable_for_cursor(*session->get_buffer_stream(id));
+        mods.stream_cursor = msh::StreamCursor{
+            id, geom::Displacement{surface_specification.hotspot_x(), surface_specification.hotspot_y()} }; 
+    }
+
     mods.input_shape = extract_input_shape_from(&surface_specification);
 
     auto const id = mf::SurfaceId(request->surface_id().value());
@@ -625,7 +688,7 @@ void mf::SessionMediator::modify_surface(
 }
 
 void mf::SessionMediator::configure_display(
-    const ::mir::protobuf::DisplayConfiguration* request,
+    ::mir::protobuf::DisplayConfiguration const* request,
     ::mir::protobuf::DisplayConfiguration* response,
     ::google::protobuf::Closure* done)
 {
@@ -634,7 +697,7 @@ void mf::SessionMediator::configure_display(
     if (session.get() == nullptr)
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
 
-    report->session_configure_display_called(session->name());
+    observer->session_configure_display_called(session->name());
 
     auto const config = unpack_and_sanitize_display_configuration(request);
     display_changer->configure(session, config);
@@ -645,6 +708,21 @@ void mf::SessionMediator::configure_display(
     done->Run();
 }
 
+void mf::SessionMediator::remove_session_configuration(
+    ::mir::protobuf::Void const* /*request*/,
+    ::mir::protobuf::Void* /*response*/,
+    ::google::protobuf::Closure* done)
+{
+    auto session = weak_session.lock();
+
+    if (session.get() == nullptr)
+        BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
+
+    display_changer->remove_session_configuration(session);
+
+    done->Run();
+}
+
 void mf::SessionMediator::set_base_display_configuration(
     mir::protobuf::DisplayConfiguration const* request,
     mir::protobuf::Void* /*response*/,
@@ -655,7 +733,7 @@ void mf::SessionMediator::set_base_display_configuration(
     if (session.get() == nullptr)
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
 
-    report->session_set_base_display_configuration_called(session->name());
+    observer->session_set_base_display_configuration_called(session->name());
 
     auto const config = unpack_and_sanitize_display_configuration(request);
     display_changer->set_base_configuration(config);
@@ -673,7 +751,7 @@ void mf::SessionMediator::preview_base_display_configuration(
     if (session.get() == nullptr)
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
 
-    report->session_preview_base_display_configuration_called(session->name());
+    observer->session_preview_base_display_configuration_called(session->name());
 
     auto const config = unpack_and_sanitize_display_configuration(&request->configuration());
     display_changer->preview_base_configuration(
@@ -694,7 +772,7 @@ void mf::SessionMediator::confirm_base_display_configuration(
     if (session.get() == nullptr)
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
 
-    report->session_confirm_base_display_configuration_called(session->name());
+    observer->session_confirm_base_display_configuration_called(session->name());
 
     auto const config = unpack_and_sanitize_display_configuration(request);
 
@@ -741,17 +819,22 @@ void mf::SessionMediator::create_screencast(
         mirror_mode = static_cast<MirMirrorMode>(parameters->mirror_mode());
 
     auto screencast_session_id = screencast->create_session(region, size, pixel_format, nbuffers, mirror_mode);
-    auto buffer = screencast->capture(screencast_session_id);
-    screencast_buffer_tracker.track_buffer(screencast_session_id, buffer.get());
+
+    if (nbuffers > 0)
+    {
+        auto buffer = screencast->capture(screencast_session_id);
+        screencast_buffer_tracker.track_buffer(screencast_session_id, buffer.get());
+        pack_protobuf_buffer(
+            *protobuf_screencast->mutable_buffer_stream()->mutable_buffer(),
+            buffer.get(),
+            msg_type);
+    }
 
     protobuf_screencast->mutable_screencast_id()->set_value(
         screencast_session_id.as_value());
-
+    protobuf_screencast->mutable_buffer_stream()->set_pixel_format(pixel_format);
     protobuf_screencast->mutable_buffer_stream()->mutable_id()->set_value(
         screencast_session_id.as_value());
-    pack_protobuf_buffer(*protobuf_screencast->mutable_buffer_stream()->mutable_buffer(),
-                         buffer.get(),
-                         msg_type);
 
     done->Run();
 }
@@ -787,6 +870,19 @@ void mf::SessionMediator::screencast_buffer(
     done->Run();
 }
 
+void mf::SessionMediator::screencast_to_buffer(
+    mir::protobuf::ScreencastRequest const* request,
+    mir::protobuf::Void*,
+    google::protobuf::Closure* done)
+{
+    auto session = weak_session.lock();
+    ScreencastSessionId const screencast_session_id{request->id().value()};
+    auto buffer = session->get_buffer(mg::BufferID{request->buffer_id()});
+    mf::ScreencastSessionId const screencast_id{request->id().value()};
+    screencast->capture(screencast_session_id, buffer);
+    done->Run();
+}
+
 void mf::SessionMediator::create_buffer_stream(
     mir::protobuf::BufferStreamParameters const* request,
     mir::protobuf::BufferStream* response,
@@ -797,7 +893,7 @@ void mf::SessionMediator::create_buffer_stream(
     if (session.get() == nullptr)
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
 
-    report->session_create_buffer_stream_called(session->name());
+    observer->session_create_buffer_stream_called(session->name());
     
     auto const usage = (request->buffer_usage() == mir_buffer_usage_hardware) ?
         mg::BufferUsage::hardware : mg::BufferUsage::software;
@@ -828,7 +924,7 @@ void mf::SessionMediator::release_buffer_stream(
     if (session.get() == nullptr)
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
 
-    report->session_release_buffer_stream_called(session->name());
+    observer->session_release_buffer_stream_called(session->name());
 
     auto const id = BufferStreamId(request->value());
 
@@ -838,11 +934,12 @@ void mf::SessionMediator::release_buffer_stream(
 }
 
 
-std::function<void(std::shared_ptr<mf::Session> const&)> mf::SessionMediator::prompt_session_connect_handler() const
+auto mf::SessionMediator::prompt_session_connect_handler(detail::PromptSessionId prompt_session_id) const
+-> std::function<void(std::shared_ptr<mf::Session> const&)>
 {
-    return [this](std::shared_ptr<mf::Session> const& session)
+    return [this, prompt_session_id](std::shared_ptr<mf::Session> const& session)
     {
-        auto prompt_session = weak_prompt_session.lock();
+        auto prompt_session = prompt_sessions.fetch(prompt_session_id);
         if (prompt_session.get() == nullptr)
             BOOST_THROW_EXCEPTION(std::logic_error("Invalid prompt session"));
 
@@ -850,15 +947,6 @@ std::function<void(std::shared_ptr<mf::Session> const&)> mf::SessionMediator::pr
     };
 }
 
-namespace
-{
-void throw_if_unsuitable_for_cursor(mf::BufferStream& stream)
-{
-    if (stream.pixel_format() != mir_pixel_format_argb_8888)
-        BOOST_THROW_EXCEPTION(std::logic_error("Only argb8888 buffer streams may currently be attached to the cursor"));
-}
-}
-
 void mf::SessionMediator::configure_cursor(
     mir::protobuf::CursorSetting const* cursor_request,
     mir::protobuf::Void* /* void_response */,
@@ -869,7 +957,7 @@ void mf::SessionMediator::configure_cursor(
     if (session.get() == nullptr)
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
 
-    report->session_configure_surface_cursor_called(session->name());
+    observer->session_configure_surface_cursor_called(session->name());
 
     auto const id = mf::SurfaceId(cursor_request->surfaceid().value());
     auto const surface = session->get_surface(id);
@@ -907,7 +995,7 @@ void mf::SessionMediator::new_fds_for_prompt_providers(
     if (session.get() == nullptr)
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
 
-    auto const connect_handler = prompt_session_connect_handler();
+    auto const connect_handler = prompt_session_connect_handler(detail::PromptSessionId(parameters->prompt_session_id()));
 
     auto const fds_requested = parameters->number();
 
@@ -994,7 +1082,7 @@ void mf::SessionMediator::platform_operation(
 
 void mf::SessionMediator::start_prompt_session(
     const ::mir::protobuf::PromptSessionParameters* request,
-    ::mir::protobuf::Void* /*response*/,
+    ::mir::protobuf::PromptSession* response,
     ::google::protobuf::Closure* done)
 {
     auto const session = weak_session.lock();
@@ -1002,21 +1090,18 @@ void mf::SessionMediator::start_prompt_session(
     if (!session)
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
 
-    if (weak_prompt_session.lock())
-        BOOST_THROW_EXCEPTION(std::runtime_error("Cannot start another prompt session"));
-
     ms::PromptSessionCreationParameters parameters;
     parameters.application_pid = request->application_pid();
 
-    report->session_start_prompt_session_called(session->name(), parameters.application_pid);
+    observer->session_start_prompt_session_called(session->name(), parameters.application_pid);
 
-    weak_prompt_session = shell->start_prompt_session_for(session, parameters);
+    response->set_id(prompt_sessions.insert(shell->start_prompt_session_for(session, parameters)).as_value());
 
     done->Run();
 }
 
 void mf::SessionMediator::stop_prompt_session(
-    const ::mir::protobuf::Void*,
+    protobuf::PromptSession const* request,
     ::mir::protobuf::Void*,
     ::google::protobuf::Closure* done)
 {
@@ -1025,14 +1110,15 @@ void mf::SessionMediator::stop_prompt_session(
     if (!session)
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
 
-    auto const prompt_session = weak_prompt_session.lock();
+    detail::PromptSessionId const id{request->id()};
+    auto const prompt_session = prompt_sessions.fetch(id);
 
     if (!prompt_session)
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid prompt session"));
 
-    weak_prompt_session.reset();
+    prompt_sessions.remove(id);
 
-    report->session_stop_prompt_session_called(session->name());
+    observer->session_stop_prompt_session_called(session->name());
 
     shell->stop_prompt_session(prompt_session);
 
@@ -1136,6 +1222,7 @@ mf::SessionMediator::unpack_and_sanitize_display_configuration(
                 static_cast<MirPixelFormat>(src.current_format());
         dest.power_mode = static_cast<MirPowerMode>(src.power_mode());
         dest.orientation = static_cast<MirOrientation>(src.orientation());
+        dest.scale = src.scale_factor();
 
         dest.gamma = {convert_string_to_gamma_curve(src.gamma_red()),
                       convert_string_to_gamma_curve(src.gamma_green()),
@@ -1157,3 +1244,25 @@ void mf::SessionMediator::destroy_screencast_sessions()
     for (auto const& id : ids_to_untrack)
         screencast_buffer_tracker.remove_session(id);
 }
+
+
+auto mf::detail::PromptSessionStore::insert(std::shared_ptr<PromptSession> const& session) -> PromptSessionId
+{
+    std::lock_guard<decltype(mutex)> lock{mutex};
+    auto const id = PromptSessionId{++next_id};
+    sessions[id] = session;
+    return id;
+}
+
+auto mf::detail::PromptSessionStore::fetch(PromptSessionId session) const
+-> std::shared_ptr<PromptSession>
+{
+    std::lock_guard<decltype(mutex)> lock{mutex};
+    return sessions[session].lock();
+}
+
+void mf::detail::PromptSessionStore::remove(PromptSessionId session)
+{
+    std::lock_guard<decltype(mutex)> lock{mutex};
+    sessions.erase(session);
+}
