@@ -19,6 +19,7 @@
 #include "mir/geometry/rectangle.h"
 #include "mir/scene/session.h"
 #include "mir_toolkit/events/surface_placement.h"
+#include "mir_toolkit/events/window_placement.h"
 #include "mir/events/event_builders.h"
 #include "mir/scene/surface.h"
 
@@ -73,14 +74,14 @@ struct Client
         std::swap(connection, source.connection);
     }
 
-    auto surface_create() const -> MirSurface*
+    auto surface_create() const -> MirWindow*
     {
-        auto spec = mir_connection_create_spec_for_normal_surface(
-            connection, 800, 600, mir_pixel_format_bgr_888);
-        auto surface = mir_surface_create_sync(spec);
-        mir_surface_spec_release(spec);
+        auto spec = mir_create_normal_window_spec(connection, 800, 600);
+        mir_window_spec_set_pixel_format(spec, mir_pixel_format_bgr_888);
+        auto window = mir_create_window_sync(spec);
+        mir_window_spec_release(spec);
 
-        return surface;
+        return window;
     }
 
     void disconnect()
@@ -167,9 +168,9 @@ TEST_F(CustomWindowManagement, surface_create_adds_surface)
 
     EXPECT_CALL(window_manager, add_surface(_,_,_));
 
-    auto const surface = client.surface_create();
+    auto const window = client.surface_create();
 
-    mir_surface_release_sync(surface);
+    mir_window_release_sync(window);
 }
 
 TEST_F(CustomWindowManagement, surface_rename_modifies_surface)
@@ -178,16 +179,16 @@ TEST_F(CustomWindowManagement, surface_rename_modifies_surface)
 
     start_server();
     auto const client = connect_client();
-    auto const surface = client.surface_create();
+    auto const window = client.surface_create();
 
     EXPECT_CALL(window_manager, modify_surface(_,_,_));
 
-    auto const spec = mir_connection_create_spec_for_changes(client.connection);
-    mir_surface_spec_set_name(spec, new_title);
-    mir_surface_apply_spec(surface, spec);
-    mir_surface_spec_release(spec);
+    auto const spec = mir_create_window_spec(client.connection);
+    mir_window_spec_set_name(spec, new_title);
+    mir_window_apply_spec(window, spec);
+    mir_window_spec_release(spec);
 
-    mir_surface_release_sync(surface);
+    mir_window_release_sync(window);
 }
 
 TEST_F(CustomWindowManagement, surface_release_removes_surface)
@@ -195,11 +196,11 @@ TEST_F(CustomWindowManagement, surface_release_removes_surface)
     start_server();
 
     auto const client = connect_client();
-    auto const surface = client.surface_create();
+    auto const window = client.surface_create();
 
     EXPECT_CALL(window_manager, remove_surface(_,_));
 
-    mir_surface_release_sync(surface);
+    mir_window_release_sync(window);
 
     Mock::VerifyAndClearExpectations(&window_manager);
 }
@@ -237,8 +238,8 @@ TEST_F(CustomWindowManagement, surface_is_associated_with_correct_client)
         EXPECT_CALL(window_manager, add_surface(WeakPtrEq(session[i]),_,_));
         EXPECT_CALL(window_manager, remove_surface(WeakPtrEq(session[i]),_));
 
-        auto const surface = client[i].surface_create();
-        mir_surface_release_sync(surface);
+        auto const window = client[i].surface_create();
+        mir_window_release_sync(window);
 
         // verify expectations for each client in turn
         Mock::VerifyAndClearExpectations(&window_manager);
@@ -252,7 +253,7 @@ TEST_F(CustomWindowManagement, state_change_requests_are_associated_with_correct
 
     const int no_of_surfaces = 17;
     std::weak_ptr<ms::Surface> server_surface[no_of_surfaces];
-    MirSurface* client_surface[no_of_surfaces] = {};
+    MirWindow* client_surface[no_of_surfaces] = {};
 
     for (int i = 0; i != no_of_surfaces; ++i)
     {
@@ -274,21 +275,21 @@ TEST_F(CustomWindowManagement, state_change_requests_are_associated_with_correct
 
     for (int i = 0; i != no_of_surfaces; ++i)
     {
-        // verify expectations for each surface in turn
+        // verify expectations for each window in turn
         Mock::VerifyAndClearExpectations(&window_manager);
 
         mt::Signal received;
 
-        EXPECT_CALL(window_manager, set_surface_attribute(_, WeakPtrEq(server_surface[i]), mir_surface_attrib_state,_))
+        EXPECT_CALL(window_manager, set_surface_attribute(_, WeakPtrEq(server_surface[i]), mir_window_attrib_state,_))
             .WillOnce(WithArg<3>(Invoke([&](int value) { received.raise(); return value; })));
 
-        mir_surface_set_state(client_surface[i], mir_surface_state_maximized);
+        mir_window_set_state(client_surface[i], mir_window_state_maximized);
 
         received.wait_for(400ms);
     }
 
-    for (auto const surface : client_surface)
-        mir_surface_release_sync(surface);
+    for (auto const window : client_surface)
+        mir_window_release_sync(window);
 }
 
 TEST_F(CustomWindowManagement, create_low_chrome_surface_from_spec)
@@ -299,11 +300,10 @@ TEST_F(CustomWindowManagement, create_low_chrome_surface_from_spec)
 
     int const width{800}, height{600};
     MirPixelFormat const format{mir_pixel_format_bgr_888};
-    auto surface_spec = mir_connection_create_spec_for_normal_surface(connection,
-                                                                      width, height,
-                                                                      format);
+    auto surface_spec = mir_create_normal_window_spec(connection, width, height);
 
-    mir_surface_spec_set_shell_chrome(surface_spec, mir_shell_chrome_low);
+    mir_window_spec_set_pixel_format(surface_spec, format);
+    mir_window_spec_set_shell_chrome(surface_spec, mir_shell_chrome_low);
 
     auto const check_add_surface = [](
         std::shared_ptr<ms::Session> const& session,
@@ -316,10 +316,10 @@ TEST_F(CustomWindowManagement, create_low_chrome_surface_from_spec)
 
     EXPECT_CALL(window_manager, add_surface(_,_,_)).WillOnce(Invoke(check_add_surface));
 
-    auto surface = mir_surface_create_sync(surface_spec);
-    mir_surface_spec_release(surface_spec);
+    auto window = mir_create_window_sync(surface_spec);
+    mir_window_spec_release(surface_spec);
 
-    mir_surface_release_sync(surface);
+    mir_window_release_sync(window);
     mir_connection_release(connection);
 }
 
@@ -331,18 +331,17 @@ TEST_F(CustomWindowManagement, apply_low_chrome_to_surface)
 
     int const width{800}, height{600};
     MirPixelFormat const format{mir_pixel_format_bgr_888};
-    auto surface_spec = mir_connection_create_spec_for_normal_surface(connection,
-                                                                      width, height,
-                                                                      format);
+    auto surface_spec = mir_create_normal_window_spec(connection, width, height);
+    mir_window_spec_set_pixel_format(surface_spec, format);
 
-    auto surface = mir_surface_create_sync(surface_spec);
-    mir_surface_spec_release(surface_spec);
+    auto window = mir_create_window_sync(surface_spec);
+    mir_window_spec_release(surface_spec);
 
-    surface_spec = mir_connection_create_spec_for_changes(connection);
+    surface_spec = mir_create_window_spec(connection);
 
     mt::Signal received;
 
-    mir_surface_spec_set_shell_chrome(surface_spec, mir_shell_chrome_low);
+    mir_window_spec_set_shell_chrome(surface_spec, mir_shell_chrome_low);
 
     auto const check_apply_surface = [&received](
         std::shared_ptr<ms::Session> const&,
@@ -355,12 +354,12 @@ TEST_F(CustomWindowManagement, apply_low_chrome_to_surface)
 
     EXPECT_CALL(window_manager, modify_surface(_,_,_)).WillOnce(Invoke(check_apply_surface));
 
-    mir_surface_apply_spec(surface, surface_spec);
-    mir_surface_spec_release(surface_spec);
+    mir_window_apply_spec(window, surface_spec);
+    mir_window_spec_release(surface_spec);
 
     EXPECT_TRUE(received.wait_for(400ms));
 
-    mir_surface_release_sync(surface);
+    mir_window_release_sync(window);
     mir_connection_release(connection);
 }
 
@@ -379,14 +378,16 @@ TEST_F(CustomWindowManagement, when_the_client_places_a_new_surface_the_request_
 
     start_server();
     auto connection = mir_connect_sync(new_connection().c_str(), __PRETTY_FUNCTION__);
-    auto surface_spec = mir_connection_create_spec_for_normal_surface(connection, width, height, format);
-    auto parent = mir_surface_create_sync(surface_spec);
-    mir_surface_spec_release(surface_spec);
+    auto surface_spec = mir_create_normal_window_spec(connection, width, height);
+    mir_window_spec_set_pixel_format(surface_spec, format);
+    auto parent = mir_create_window_sync(surface_spec);
+    mir_window_spec_release(surface_spec);
 
-    surface_spec = mir_connection_create_spec_for_tip(
-        connection, width, height, format, parent, &dummy_rect, mir_edge_attachment_any);
+    surface_spec = mir_create_tip_window_spec(
+        connection, width, height, parent, &dummy_rect, mir_edge_attachment_any);
+    mir_window_spec_set_pixel_format(surface_spec, format);
 
-    mir_surface_spec_set_placement(
+    mir_window_spec_set_placement(
         surface_spec, &aux_rect, rect_gravity, surface_gravity, placement_hints, offset_dx, offset_dy);
 
     mt::Signal received;
@@ -432,13 +433,13 @@ TEST_F(CustomWindowManagement, when_the_client_places_a_new_surface_the_request_
 
     EXPECT_CALL(window_manager, add_surface(_,_,_)).WillOnce(Invoke(check_placement));
 
-    auto child = mir_surface_create_sync(surface_spec);
-    mir_surface_spec_release(surface_spec);
+    auto child = mir_create_window_sync(surface_spec);
+    mir_window_spec_release(surface_spec);
 
     EXPECT_TRUE(received.wait_for(400ms));
 
-    mir_surface_release_sync(child);
-    mir_surface_release_sync(parent);
+    mir_window_release_sync(child);
+    mir_window_release_sync(parent);
     mir_connection_release(connection);
 }
 
@@ -457,17 +458,19 @@ TEST_F(CustomWindowManagement, when_the_client_places_an_existing_surface_the_re
 
     start_server();
     auto connection = mir_connect_sync(new_connection().c_str(), __PRETTY_FUNCTION__);
-    auto surface_spec = mir_connection_create_spec_for_normal_surface(connection, width, height, format);
-    auto parent = mir_surface_create_sync(surface_spec);
-    mir_surface_spec_release(surface_spec);
+    auto surface_spec = mir_create_normal_window_spec(connection, width, height);
+    mir_window_spec_set_pixel_format(surface_spec, format);
+    auto parent = mir_create_window_sync(surface_spec);
+    mir_window_spec_release(surface_spec);
 
-    surface_spec = mir_connection_create_spec_for_menu(
-        connection, width, height, format, parent, &dummy_rect, mir_edge_attachment_any);
-    auto child = mir_surface_create_sync(surface_spec);
-    mir_surface_spec_release(surface_spec);
+    surface_spec = mir_create_menu_window_spec(
+        connection, width, height, parent, &dummy_rect, mir_edge_attachment_any);
+    mir_window_spec_set_pixel_format(surface_spec, format);
+    auto child = mir_create_window_sync(surface_spec);
+    mir_window_spec_release(surface_spec);
 
-    surface_spec = mir_connection_create_spec_for_changes(connection);
-    mir_surface_spec_set_placement(
+    surface_spec = mir_create_window_spec(connection);
+    mir_window_spec_set_placement(
         surface_spec, &aux_rect, rect_gravity, surface_gravity, placement_hints, offset_dx, offset_dy);
 
     mt::Signal received;
@@ -511,13 +514,13 @@ TEST_F(CustomWindowManagement, when_the_client_places_an_existing_surface_the_re
         };
 
     EXPECT_CALL(window_manager, modify_surface(_,_,_)).WillOnce(Invoke(check_placement));
-    mir_surface_apply_spec(child, surface_spec);
-    mir_surface_spec_release(surface_spec);
+    mir_window_apply_spec(child, surface_spec);
+    mir_window_spec_release(surface_spec);
 
     EXPECT_TRUE(received.wait_for(400ms));
 
-    mir_surface_release_sync(child);
-    mir_surface_release_sync(parent);
+    mir_window_release_sync(child);
+    mir_window_release_sync(parent);
     mir_connection_release(connection);
 }
 
@@ -527,12 +530,12 @@ struct PlacementCheck
 {
     PlacementCheck(MirRectangle const& placement) : expected_rect{placement} {}
 
-    void check(MirSurfacePlacementEvent const* placement_event)
+    void check(MirWindowPlacementEvent const* placement_event)
     {
-        EXPECT_THAT(mir_surface_placement_get_relative_position(placement_event).top, Eq(expected_rect.top));
-        EXPECT_THAT(mir_surface_placement_get_relative_position(placement_event).left, Eq(expected_rect.left));
-        EXPECT_THAT(mir_surface_placement_get_relative_position(placement_event).height, Eq(expected_rect.height));
-        EXPECT_THAT(mir_surface_placement_get_relative_position(placement_event).width, Eq(expected_rect.width));
+        EXPECT_THAT(mir_window_placement_get_relative_position(placement_event).top, Eq(expected_rect.top));
+        EXPECT_THAT(mir_window_placement_get_relative_position(placement_event).left, Eq(expected_rect.left));
+        EXPECT_THAT(mir_window_placement_get_relative_position(placement_event).height, Eq(expected_rect.height));
+        EXPECT_THAT(mir_window_placement_get_relative_position(placement_event).width, Eq(expected_rect.width));
 
         received.raise();
     }
@@ -547,11 +550,11 @@ private:
     mt::Signal received;
 };
 
-void surface_placement_event_callback(MirSurface* /*surface*/, MirEvent const* event, void* context)
+void window_placement_event_callback(MirWindow* /*window*/, MirEvent const* event, void* context)
 {
-    if (mir_event_get_type(event) == mir_event_type_surface_placement)
+    if (mir_event_get_type(event) == mir_event_type_window_placement)
     {
-        auto const placement_event = mir_event_get_surface_placement_event(event);
+        auto const placement_event = mir_event_get_window_placement_event(event);
         static_cast<PlacementCheck*>(context)->check(placement_event);
     }
 }
@@ -584,13 +587,14 @@ TEST_F(CustomWindowManagement, when_the_window_manager_places_a_surface_the_noti
     auto connection = mir_connect_sync(new_connection().c_str(), __PRETTY_FUNCTION__);
 
     PlacementCheck placement_check{placement};
-    auto surface_spec = mir_connection_create_spec_for_normal_surface(connection, width, height, format);
-    mir_surface_spec_set_event_handler(surface_spec, &surface_placement_event_callback, &placement_check);
-    auto surface = mir_surface_create_sync(surface_spec);
-    mir_surface_spec_release(surface_spec);
+    auto surface_spec = mir_create_normal_window_spec(connection, width, height);
+    mir_window_spec_set_pixel_format(surface_spec, format);
+    mir_window_spec_set_event_handler(surface_spec, &window_placement_event_callback, &placement_check);
+    auto window = mir_create_window_sync(surface_spec);
+    mir_window_spec_release(surface_spec);
 
     scene_surface->placed_relative(placement_);
 
-    mir_surface_release_sync(surface);
+    mir_window_release_sync(window);
     mir_connection_release(connection);
 }
