@@ -23,6 +23,8 @@
 #include "src/server/scene/application_session.h"
 #include "src/server/frontend/event_sender.h"
 #include "src/server/frontend/protobuf_buffer_packer.h"
+#include "src/server/input/builtin_cursor_images.h"
+#include "src/server/input/default-theme.h"
 #include "mir/graphics/display.h"
 #include "mir/graphics/display_configuration.h"
 #include "mir/graphics/platform.h"
@@ -32,6 +34,7 @@
 #include "mir/graphics/buffer_id.h"
 #include "mir/input/cursor_images.h"
 #include "mir/graphics/platform_ipc_operations.h"
+#include "mir/scene/coordinate_translator.h"
 #include "src/server/scene/basic_surface.h"
 #include "mir/test/doubles/mock_display.h"
 #include "mir/test/doubles/mock_display_changer.h"
@@ -65,6 +68,8 @@
 #include "mir/frontend/connector.h"
 #include "mir/frontend/event_sink.h"
 #include "mir/cookie/authority.h"
+#include "mir/input/mir_input_config.h"
+#include "mir/input/mir_input_config_serialization.h"
 #include "mir_protobuf.pb.h"
 #include "mir_protobuf_wire.pb.h"
 
@@ -160,17 +165,8 @@ public:
     std::shared_ptr<mtd::MockBufferStream> create_mock_stream(mf::BufferStreamId id)
     {
         mock_streams[id] = std::make_shared<testing::NiceMock<mtd::MockBufferStream>>();
-        auto buffer1 = std::make_shared<mtd::StubBuffer>();
-        auto buffer2 = std::make_shared<mtd::StubBuffer>();
-        ON_CALL(*mock_streams[id], swap_buffers(testing::_,testing::_))
-            .WillByDefault(testing::Invoke(
-            [buffer1, buffer2](mg::Buffer* b, std::function<void(mg::Buffer* new_buffer)> complete)
-            {
-                if ((!b) || (b == buffer1.get()))
-                    complete(buffer2.get());
-                if (b == buffer2.get())
-                    complete(buffer1.get());
-            }));
+        ON_CALL(*mock_streams[id], pixel_format())
+            .WillByDefault(Return(mir_pixel_format_argb_8888));
         return mock_streams[id];
     }
 
@@ -206,6 +202,18 @@ public:
         return mg::BufferID{3};
     }
 
+    mg::BufferID create_buffer(geom::Size, MirPixelFormat) override
+    {
+        buffer_count++;
+        return mg::BufferID{3};
+    }
+
+    mg::BufferID create_buffer(geom::Size, uint32_t, uint32_t) override
+    {
+        native_buffer_count++;
+        return mg::BufferID{3};
+    }
+
     MOCK_METHOD1(destroy_buffer_stream, void(mf::BufferStreamId));
     MOCK_METHOD1(destroy_buffer, void(mg::BufferID));
 
@@ -225,6 +233,7 @@ public:
     int last_stream_id = 0;
     int last_surface_id = 0;
     int buffer_count = 0;
+    int native_buffer_count = 0;
     int destroy_buffers = 0;
 };
 
@@ -240,6 +249,18 @@ struct StubScreencast : mtd::NullScreencast
     mtd::StubBuffer stub_buffer;
 };
 
+struct NullCoordinateTranslator : ms::CoordinateTranslator
+{
+    geom::Point surface_to_screen(std::shared_ptr<mf::Surface>, int32_t x, int32_t y) override
+    {
+        return {x, y};
+    }
+    bool translation_supported() const override
+    {
+        return true;
+    }
+};
+
 struct SessionMediator : public ::testing::Test
 {
     SessionMediator()
@@ -256,7 +277,9 @@ struct SessionMediator : public ::testing::Test
             surface_pixel_formats, report,
             std::make_shared<mtd::NullEventSinkFactory>(),
             std::make_shared<mtd::NullMessageSender>(),
-            resource_cache, stub_screencast, &connector, nullptr, nullptr,
+            resource_cache, stub_screencast, &connector,
+            std::make_shared<mi::BuiltinCursorImages>(),
+            std::make_shared<NullCoordinateTranslator>(),
             std::make_shared<mtd::NullANRDetector>(),
             mir::cookie::Authority::create(),
             mt::fake_shared(mock_hub)}
@@ -282,7 +305,8 @@ struct SessionMediator : public ::testing::Test
             std::make_shared<mtd::NullEventSinkFactory>(),
             std::make_shared<mtd::NullMessageSender>(),
             resource_cache, std::make_shared<mtd::NullScreencast>(),
-            nullptr, nullptr, nullptr,
+            nullptr, nullptr, 
+            std::make_shared<NullCoordinateTranslator>(),
             std::make_shared<mtd::NullANRDetector>(),
             mir::cookie::Authority::create(),
             mt::fake_shared(mock_hub));
@@ -296,7 +320,8 @@ struct SessionMediator : public ::testing::Test
             surface_pixel_formats, report,
             std::make_shared<mtd::NullEventSinkFactory>(),
             std::make_shared<mtd::NullMessageSender>(),
-            resource_cache, screencast, &connector, nullptr, nullptr,
+            resource_cache, screencast, &connector, nullptr,
+            std::make_shared<NullCoordinateTranslator>(),
             std::make_shared<mtd::NullANRDetector>(),
             mir::cookie::Authority::create(),
             mt::fake_shared(mock_hub));
@@ -308,7 +333,7 @@ struct SessionMediator : public ::testing::Test
     std::shared_ptr<testing::NiceMock<mtd::MockShell>> const shell;
     std::shared_ptr<mf::DisplayChanger> const graphics_changer;
     std::vector<MirPixelFormat> const surface_pixel_formats;
-    std::shared_ptr<mf::SessionMediatorReport> const report;
+    std::shared_ptr<mf::SessionMediatorObserver> const report;
     std::shared_ptr<mf::ResourceCache> const resource_cache;
     std::shared_ptr<StubScreencast> const stub_screencast;
     std::shared_ptr<NiceMock<StubbedSession>> const stubbed_session;
@@ -352,7 +377,8 @@ TEST_F(SessionMediator, connect_calls_connect_handler)
         surface_pixel_formats, report,
         std::make_shared<mtd::NullEventSinkFactory>(),
         std::make_shared<mtd::NullMessageSender>(),
-        resource_cache, stub_screencast, context, nullptr, nullptr,
+        resource_cache, stub_screencast, context, nullptr,
+        std::make_shared<NullCoordinateTranslator>(),
         std::make_shared<mtd::NullANRDetector>(),
         mir::cookie::Authority::create(),
         mt::fake_shared(mock_hub)};
@@ -543,12 +569,16 @@ TEST_F(SessionMediator, fully_packs_buffer_for_create_screencast)
     mp::ScreencastParameters screencast_parameters;
     mp::Screencast screencast;
     auto const& stub_buffer = stub_screencast->stub_buffer;
+    screencast_parameters.set_pixel_format(stub_buffer.pixel_format());
 
     EXPECT_CALL(mock_ipc_operations, pack_buffer(_, Ref(stub_buffer), mg::BufferIpcMsgType::full_msg));
 
     mediator.create_screencast(&screencast_parameters,
                                &screencast, null_callback.get());
-    EXPECT_EQ(stub_buffer.id().as_value(), screencast.buffer_stream().buffer().buffer_id());
+    EXPECT_EQ(static_cast<int>(stub_buffer.id().as_value()),
+              screencast.buffer_stream().buffer().buffer_id());
+    //LP: #1662997
+    EXPECT_THAT(screencast.buffer_stream().pixel_format(), Eq(stub_buffer.pixel_format()));
 }
 
 TEST_F(SessionMediator, eventually_partially_packs_screencast_buffer)
@@ -572,10 +602,11 @@ TEST_F(SessionMediator, eventually_partially_packs_screencast_buffer)
 
     mediator.screencast_buffer(&screencast_id,
                                &protobuf_buffer, null_callback.get());
-    EXPECT_EQ(stub_buffer.id().as_value(), protobuf_buffer.buffer_id());
+    EXPECT_EQ(static_cast<int>(stub_buffer.id().as_value()),
+              protobuf_buffer.buffer_id());
 }
 
-TEST_F(SessionMediator, partially_packs_next_buffer_after_creating_screencast)
+TEST_F(SessionMediator, partially_packs_swap_buffers_after_creating_screencast)
 {
     using namespace testing;
 
@@ -675,17 +706,17 @@ TEST_F(SessionMediator, arranges_bufferstreams_via_shell)
     mediator.modify_surface(&mods, &null, null_callback.get());
 }
 
-TEST_F(SessionMediator, allocates_from_the_session)
+TEST_F(SessionMediator, allocates_software_buffers_from_the_session)
 {
     using namespace testing;
-    auto num_requests = 3u;
+    auto num_requests = 3;
     mp::Void null;
     mp::BufferStreamId id;
     id.set_value(0);
     mp::BufferAllocation request;
     *request.mutable_id() = id;
     mg::BufferProperties properties(geom::Size{34, 84}, mir_pixel_format_abgr_8888, mg::BufferUsage::hardware);
-    for(auto i = 0u; i < num_requests; i++)
+    for(auto i = 0; i < num_requests; i++)
     {
         auto buffer_request = request.add_buffer_requests();
         buffer_request->set_width(properties.size.width.as_int());
@@ -701,17 +732,41 @@ TEST_F(SessionMediator, allocates_from_the_session)
     EXPECT_THAT(stubbed_session->num_alloc_requests(), Eq(num_requests));
 }
 
+TEST_F(SessionMediator, allocates_native_buffers_from_the_session)
+{
+    using namespace testing;
+    geom::Size const size { 1029, 10302 };
+    auto native_flags = 24u;
+    auto native_format = 124u;
+    mp::Void null;
+    mp::BufferStreamId id;
+    id.set_value(0);
+    mp::BufferAllocation request;
+    *request.mutable_id() = id;
+    auto buffer_request = request.add_buffer_requests();
+    buffer_request->set_width(size.width.as_int());
+    buffer_request->set_height(size.height.as_int());
+    buffer_request->set_native_format(native_format);
+    buffer_request->set_flags(native_flags);
+
+    mediator.connect(&connect_parameters, &connection, null_callback.get());
+    mediator.create_surface(&surface_parameters, &surface_response, null_callback.get());
+
+    mediator.allocate_buffers(&request, &null, null_callback.get());
+    EXPECT_THAT(stubbed_session->native_buffer_count, Eq(1));
+}
+
 TEST_F(SessionMediator, removes_buffer_from_the_session)
 {
     using namespace testing;
-    auto num_requests = 3u;
+    auto num_requests = 3;
     mp::Void null;
     mp::BufferStreamId id;
     id.set_value(0);
     mp::BufferRelease request;
     *request.mutable_id() = id;
     auto buffer_id = 442u;
-    for(auto i = 0u; i < num_requests; i++)
+    for(auto i = 0; i < num_requests; i++)
     {
         auto buffer_request = request.add_buffers();
         buffer_request->set_buffer_id(buffer_id);
@@ -724,40 +779,6 @@ TEST_F(SessionMediator, removes_buffer_from_the_session)
     EXPECT_THAT(stubbed_session->num_destroy_requests(), Eq(num_requests));
 }
 
-TEST_F(SessionMediator, doesnt_mind_swap_buffers_returning_nullptr_in_submit)
-{
-    using namespace testing;
-    auto mock_stream = stubbed_session->mock_stream_at(mf::BufferStreamId{0});
-    ON_CALL(*mock_stream, swap_buffers(_,_))
-        .WillByDefault(InvokeArgument<1>(nullptr));
-    auto buffer1 = std::make_shared<mtd::StubBuffer>();
-    mf::SessionMediator mediator{
-        shell, mt::fake_shared(mock_ipc_operations), graphics_changer,
-        surface_pixel_formats, report,
-        std::make_shared<mtd::NullEventSinkFactory>(),
-        std::make_shared<mtd::NullMessageSender>(),
-        resource_cache, stub_screencast, nullptr, nullptr, nullptr,
-        std::make_shared<mtd::NullANRDetector>(),
-        mir::cookie::Authority::create(),
-        mt::fake_shared(mock_hub)};
-
-    mp::Void null;
-    mp::BufferRequest request;
-
-    mediator.connect(&connect_parameters, &connection, null_callback.get());
-    mediator.create_surface(&surface_parameters, &surface_response, null_callback.get());
-
-    request.mutable_id()->set_value(surface_response.id().value());
-    request.mutable_buffer()->set_buffer_id(buffer1->id().as_value());
-
-    InSequence seq;
-    EXPECT_CALL(mock_ipc_operations, unpack_buffer(_,_));
-    EXPECT_CALL(*mock_stream, swap_buffers(_,_))
-        .WillOnce(InvokeArgument<1>(nullptr));
-
-    mediator.submit_buffer(&request, &null, null_callback.get());
-}
-
 TEST_F(SessionMediator, configures_swap_intervals_on_streams)
 {
     using namespace testing;
@@ -841,7 +862,8 @@ TEST_F(SessionMediator, events_sent_before_surface_creation_reply_are_buffered)
         shell, mt::fake_shared(mock_ipc_operations), graphics_changer,
         surface_pixel_formats, report, sink_factory,
         mock_sender,
-        resource_cache, stub_screencast, nullptr, nullptr, nullptr,
+        resource_cache, stub_screencast, nullptr, nullptr,
+        std::make_shared<NullCoordinateTranslator>(),
         std::make_shared<mtd::NullANRDetector>(),
         mir::cookie::Authority::create(),
         mt::fake_shared(mock_hub)};
@@ -867,19 +889,6 @@ TEST_F(SessionMediator, events_sent_before_surface_creation_reply_are_buffered)
         google::protobuf::NewCallback(&send_non_event, mock_sender));
 }
 
-TEST_F(SessionMediator, doesnt_inadventently_set_buffer_field_when_theres_no_buffer)
-{
-    mp::Void null;
-    mp::BufferStreamParameters stream_request;
-    mp::BufferStream stream_response;
-    auto stream = stubbed_session->mock_stream_at(mf::BufferStreamId{0});
-    ON_CALL(*stream, swap_buffers(nullptr,testing::_))
-        .WillByDefault(testing::InvokeArgument<1>(nullptr));
-    mediator.connect(&connect_parameters, &connection, null_callback.get());
-    mediator.create_buffer_stream(&stream_request, &stream_response, null_callback.get());
-    EXPECT_FALSE(stream_response.has_buffer());
-}
-
 TEST_F(SessionMediator, sets_base_display_configuration)
 {
     using namespace testing;
@@ -966,8 +975,9 @@ TEST_F(SessionMediator, connect_sends_input_devices_at_seat)
             }));
 
     mediator.connect(&connect_parameters, &connection, null_callback.get());
+    auto received_conf = mir::input::deserialize_input_config(connection.input_configuration());
 
-    EXPECT_THAT(connection.input_devices(), mt::InputDevicesMatch(devices));
+    EXPECT_THAT(received_conf, mt::InputConfigurationMatches(devices));
 }
 
 TEST_F(SessionMediator, disconnect_removes_orphaned_screencast_sessions)
@@ -1064,3 +1074,106 @@ TEST_F(SessionMediator, releases_buffers_of_unknown_buffer_stream_does_not_throw
         mediator.release_buffers(&release_buffer, &null, null_callback.get());
         );
 }
+
+MATCHER_P3(CursorIs, id_value, x_value, y_value, "cursor configuration match")
+{
+    if (!arg.stream_cursor.is_set())
+        return false;
+    auto& cursor = arg.stream_cursor.value();
+    EXPECT_THAT(cursor.hotspot.dx.as_int(), testing::Eq(x_value));
+    EXPECT_THAT(cursor.hotspot.dy.as_int(), testing::Eq(y_value));
+    EXPECT_THAT(cursor.stream_id.as_value(), testing::Eq(id_value));
+    return !(::testing::Test::HasFailure());
+}
+
+MATCHER(CursorImageIsSetNull, "cursor configuration match")
+{
+    if (!arg.cursor_image.is_set())
+        return false;
+    EXPECT_THAT(arg.cursor_image.value(), testing::Eq(nullptr));
+    return !(::testing::Test::HasFailure());
+}
+MATCHER(CursorImageIsSetNotNull, "cursor configuration match")
+{
+    if (!arg.cursor_image.is_set())
+        return false;
+    EXPECT_THAT(arg.cursor_image.value(), testing::Ne(nullptr));
+    return !(::testing::Test::HasFailure());
+}
+
+TEST_F(SessionMediator, arranges_cursors_via_shell)
+{
+    using namespace testing;
+    mp::Void null;
+    mp::SurfaceModifications mods;
+    mp::BufferStreamParameters stream_request;
+    mp::BufferStream stream;
+
+    auto spec = mods.mutable_surface_specification();
+    mediator.connect(&connect_parameters, &connection, null_callback.get());
+    mediator.create_surface(&surface_parameters, &surface_response, null_callback.get());
+    mediator.create_buffer_stream(&stream_request, &stream, null_callback.get());
+    spec->mutable_cursor_id()->set_value(stream.id().value());
+    spec->set_hotspot_x(-1);
+    spec->set_hotspot_y(2);
+    EXPECT_CALL(*shell, modify_surface(_,
+        mf::SurfaceId{surface_response.id().value()},
+        CursorIs(stream.id().value(), spec->hotspot_x(), spec->hotspot_y())));
+    mediator.modify_surface(&mods, &null, null_callback.get());
+}
+
+TEST_F(SessionMediator, arranges_named_cursors_via_shell)
+{
+    mp::Void null;
+    mp::SurfaceModifications mods;
+    auto spec = mods.mutable_surface_specification();
+    mediator.connect(&connect_parameters, &connection, null_callback.get());
+    mediator.create_surface(&surface_parameters, &surface_response, null_callback.get());
+    spec->set_cursor_name("cursor_gigante");
+
+    testing::Sequence seq;
+    EXPECT_CALL(*shell, modify_surface(_,
+        mf::SurfaceId{surface_response.id().value()}, CursorImageIsSetNull()))
+        .InSequence(seq);
+    EXPECT_CALL(*shell, modify_surface(_,
+        mf::SurfaceId{surface_response.id().value()}, CursorImageIsSetNotNull()))
+        .InSequence(seq);
+    mediator.modify_surface(&mods, &null, null_callback.get());
+
+    ASSERT_THAT(cursor_data.begin(), Ne(cursor_data.end()));
+    spec->set_cursor_name(cursor_data.begin()->name);
+    mediator.modify_surface(&mods, &null, null_callback.get());
+}
+
+TEST_F(SessionMediator, screencast_to_buffer_looks_up_and_fills_appropriate_buffer)
+{
+    mp::Void null;
+    mp::ScreencastParameters screencast_parameters;
+    screencast_parameters.set_num_buffers(0);
+    mp::Screencast screencast;
+    mp::BufferAllocation request;
+    auto buffer_request = request.add_buffer_requests();
+    buffer_request->set_width(100);
+    buffer_request->set_height(129);
+    buffer_request->set_pixel_format(mir_pixel_format_abgr_8888);
+    buffer_request->set_buffer_usage(mir_buffer_usage_hardware);
+    int buffer_id = 3;
+    mf::ScreencastSessionId screencast_id{7};
+    auto mock_screencast = std::make_shared<NiceMock<mtd::MockScreencast>>();
+
+    EXPECT_CALL(*mock_screencast, create_session(_,_,_,_,_))
+        .WillOnce(Return(screencast_id));
+    EXPECT_CALL(*mock_screencast, capture(screencast_id, _));
+
+    auto mediator = create_session_mediator_with_screencast(mock_screencast);
+    mediator->connect(&connect_parameters, &connection, null_callback.get());
+    mediator->allocate_buffers(&request, &null, null_callback.get());
+    mediator->create_screencast(&screencast_parameters, &screencast, null_callback.get());
+
+    mp::ScreencastRequest screencast_request;
+
+    screencast_request.mutable_id()->set_value(screencast_id.as_value());
+    screencast_request.set_buffer_id(buffer_id);
+
+    mediator->screencast_to_buffer(&screencast_request, &null, null_callback.get());
+}
